'! A function to parse the contents of a given INI file and return them as a
'! dictionary. Key/value pairs must have the form "KEY = VALUE" (without the
'! double quotes). Leading or trailing whitespace is automatically removed from
'! both key and value. A semicolon starts a comment. Comments are removed
'! prior to evaluating a line, i.e. neither section titles, nor keys or values
'! can contain semicolons!
'!
'! @author  Ansgar Wiechers <ansgar.wiechers@planetcobalt.net>
'! @date    2010-10-06
'! @version 1.0

Private Const EMPTY_SECTION_NAME   = 8000
Private Const INVALID_SECTION_NAME = 8001
Private Const INVALID_PARAM_NAME   = 8002
Private Const PARAM_EXPECTED       = 8003

'! Parse the contents of an INI file and return them as a dictionary. Each
'! section of the INI file is a key in the dictionary. Each key's value is a
'! sub-dictionary that contains the parameters of the respective section as
'! key/value pairs. Parameters that were not grouped in a named section are
'! stored in a dictionary identified by the key with the empty string ("").
'!
'! @param  filename  File object of the INI file (opened for reading)
'! @return A dictionary of dictionaries representing the sections and parameter
'!         names and values of the given INI file.
'!
'! @raise Empty section name (8000)
'! @raise Invalid section name (8001)
'! @raise Invalid parameter name (8002)
'! @raise Parameter definition expected (8003)
Function ParseIni(filename)
	Dim fso, config, file, section, line, param

	Set ParseIni = Nothing

	Set config = CreateObject("Scripting.Dictionary")
	section = ""

	Set fso = CreateObject("Scripting.FileSystemObject")
	Set file = fso.OpenTextFile(filename, 1, False) ' open file for reading
	Do While Not file.AtEndOfStream
		line = Trim(Replace(file.ReadLine, vbTab, " "))
		If InStr(line, ";") > 0 Then line = Trim(Left(line, InStr(line, ";") - 1)) ' remove comments
		If line <> "" Then
			If Left(line, 1) = "[" And Right(line, 1) = "]" Then
				' line is a section name
				section = Trim(Mid(line, 2, Len(line) - 2))
				If section = "" Then Err.Raise EMPTY_SECTION_NAME, WScript.ScriptName, "Parse Error: section name is empty string."
				If config.Exists(section) Then Err.Raise INVALID_SECTION_NAME, WScript.ScriptName, "Parse Error: duplicate section name '" & name & "'."
				config.Add section, CreateObject("Scripting.Dictionary")
			ElseIf InStr(line, "=") > 0 Then
				' line is a parameter line
				If section = "" And Not config.Exists(section) Then config.Add section, CreateObject("Scripting.Dictionary")
				param = Split(line, "=", 2)
				param(0) = Trim(param(0))
				param(1) = Trim(param(1))
				If param(0) = "" Then Err.Raise INVALID_PARAM_NAME, WScript.ScriptName, "Parse Error: invalid parameter name '" & param(0) & "'."
				If param(1) = "" Then param(1) = True
				config(section).Add param(0), param(1)
			Else
				' line is neither parameter nor section name, thus invalid
				Err.Raise PARAM_EXPECTED, WScript.ScriptName, "Parse Error: expected parameter definition in line '" & line & "'."
			End If
		End If
	Loop
	file.Close

	Set ParseIni = config
End Function
